@@ -18,8 +18,13 @@
  */
 
 #include "includes.h"
+#include "system/filesys.h"
+#include "smbd/smbd.h"
 #include "librpc/gen_ndr/xattr.h"
 #include "librpc/gen_ndr/ndr_xattr.h"
+#include "../librpc/gen_ndr/ndr_netlogon.h"
+#include "dbwrap.h"
+#include "util_tdb.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_VFS
@@ -47,15 +52,14 @@ static NTSTATUS xattr_tdb_pull_attrs(TALLOC_CTX *mem_ctx,
 
 	blob = data_blob_const(data->dptr, data->dsize);
 
-	ndr_err = ndr_pull_struct_blob(
-		&blob, result, NULL, result,
+	ndr_err = ndr_pull_struct_blob(&blob, result, result,
 		(ndr_pull_flags_fn_t)ndr_pull_tdb_xattrs);
 
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
 		DEBUG(0, ("ndr_pull_tdb_xattrs failed: %s\n",
 			  ndr_errstr(ndr_err)));
 		TALLOC_FREE(result);
-		return ndr_map_error2ntstatus(ndr_err);;
+		return ndr_map_error2ntstatus(ndr_err);
 	}
 
 	*presult = result;
@@ -73,14 +77,13 @@ static NTSTATUS xattr_tdb_push_attrs(TALLOC_CTX *mem_ctx,
 	DATA_BLOB blob;
 	enum ndr_err_code ndr_err;
 
-	ndr_err = ndr_push_struct_blob(
-		&blob, mem_ctx, NULL, attribs,
+	ndr_err = ndr_push_struct_blob(&blob, mem_ctx, attribs,
 		(ndr_push_flags_fn_t)ndr_push_tdb_xattrs);
 
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
 		DEBUG(0, ("ndr_push_tdb_xattrs failed: %s\n",
 			  ndr_errstr(ndr_err)));
-		return ndr_map_error2ntstatus(ndr_err);;
+		return ndr_map_error2ntstatus(ndr_err);
 	}
 
 	*data = make_tdb_data(blob.data, blob.length);
@@ -724,7 +727,7 @@ static void close_xattr_db(void **data)
 static int xattr_tdb_connect(vfs_handle_struct *handle, const char *service,
 			  const char *user)
 {
-	fstring sname;
+	char *sname = NULL;
 	int res, snum;
 	struct db_context *db;
 
@@ -733,9 +736,8 @@ static int xattr_tdb_connect(vfs_handle_struct *handle, const char *service,
 		return res;
 	}
 
-	fstrcpy(sname, service);
-	snum = find_service(sname);
-	if (snum == -1) {
+	snum = find_service(talloc_tos(), service, &sname);
+	if (snum == -1 || sname == NULL) {
 		/*
 		 * Should not happen, but we should not fail just *here*.
 		 */
