@@ -148,7 +148,7 @@ static NTSTATUS ntvfs_map_open_finish(struct ntvfs_module_context *ntvfs,
 	time_t write_time = 0;
 	uint32_t set_size = 0;
 	union smb_setfileinfo *sf;
-	uint_t state;
+	unsigned int state;
 
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
@@ -284,6 +284,7 @@ static NTSTATUS map_openx_open(uint16_t flags, uint16_t open_mode,
 			       union smb_open *io2)
 {
 	io2->generic.in.create_options = NTCREATEX_OPTIONS_NON_DIRECTORY_FILE;
+	io2->generic.in.private_flags = 0;
 
 	if (flags & OPENX_FLAGS_REQUEST_OPLOCK) {
 		io2->generic.in.flags |= NTCREATEX_FLAGS_REQUEST_OPLOCK;
@@ -327,7 +328,7 @@ static NTSTATUS map_openx_open(uint16_t flags, uint16_t open_mode,
 		break;
 	case OPENX_MODE_DENY_DOS:
 		/* DENY_DOS is quite strange - it depends on the filename! */
-		io2->generic.in.create_options |= 
+		io2->generic.in.private_flags |=
 			NTCREATEX_OPTIONS_PRIVATE_DENY_DOS;
 		if (is_exe_filename(fname)) {
 			io2->generic.in.share_access = 
@@ -342,7 +343,7 @@ static NTSTATUS map_openx_open(uint16_t flags, uint16_t open_mode,
 		}
 		break;
 	case OPENX_MODE_DENY_FCB:
-		io2->generic.in.create_options |= NTCREATEX_OPTIONS_PRIVATE_DENY_FCB;
+		io2->generic.in.private_flags |= NTCREATEX_OPTIONS_PRIVATE_DENY_FCB;
 		io2->generic.in.share_access = NTCREATEX_SHARE_ACCESS_NONE;
 		break;
 	default:
@@ -515,7 +516,7 @@ NTSTATUS ntvfs_map_open(struct ntvfs_module_context *ntvfs,
 			io2->generic.in.flags = 0;
 			break;
 		}
-		io2->generic.in.root_fid	= 0;
+		io2->generic.in.root_fid.fnum	= 0;
 		io2->generic.in.access_mask	= io->smb2.in.desired_access;
 		io2->generic.in.alloc_size	= io->smb2.in.alloc_size;
 		io2->generic.in.file_attr	= io->smb2.in.file_attributes;
@@ -528,6 +529,7 @@ NTSTATUS ntvfs_map_open(struct ntvfs_module_context *ntvfs,
 		io2->generic.in.sec_desc	= io->smb2.in.sec_desc;
 		io2->generic.in.ea_list		= &io->smb2.in.eas;
 		io2->generic.in.query_maximal_access = io->smb2.in.query_maximal_access; 
+		io2->generic.in.private_flags	= 0;
 
 		/* we don't support timewarp yet */
 		if (io->smb2.in.timewarp != 0) {
@@ -560,31 +562,14 @@ done:
 
 
 /* 
-   NTVFS fsinfo generic to any mapper
+   NTVFS any to fsinfo mapper
 */
-NTSTATUS ntvfs_map_fsinfo(struct ntvfs_module_context *ntvfs,
-				   struct ntvfs_request *req,
-				   union smb_fsinfo *fs)
+static NTSTATUS ntvfs_map_fsinfo_finish(struct ntvfs_module_context *ntvfs,
+				      struct ntvfs_request *req,
+				      union smb_fsinfo *fs,
+				      union smb_fsinfo *fs2,
+				      NTSTATUS status)
 {
-	NTSTATUS status;
-	union smb_fsinfo *fs2;
-
-	fs2 = talloc(req, union smb_fsinfo);
-	if (fs2 == NULL) {
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	if (fs->generic.level == RAW_QFS_GENERIC) {
-		return NT_STATUS_INVALID_LEVEL;
-	}
-	
-	/* only used by the simple backend, which doesn't do async */
-	req->async_states->state &= ~NTVFS_ASYNC_STATE_MAY_ASYNC;
-
-	/* ask the backend for the generic info */
-	fs2->generic.level = RAW_QFS_GENERIC;
-
-	status = ntvfs->ops->fsinfo(ntvfs, req, fs2);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
@@ -596,7 +581,7 @@ NTSTATUS ntvfs_map_fsinfo(struct ntvfs_module_context *ntvfs,
 
 	case RAW_QFS_DSKATTR: {
 		/* map from generic to DSKATTR */
-		uint_t bpunit = 64;
+		unsigned int bpunit = 64;
 
 		/* we need to scale the sizes to fit */
 		for (bpunit=64; bpunit<0x10000; bpunit *= 2) {
@@ -687,6 +672,38 @@ NTSTATUS ntvfs_map_fsinfo(struct ntvfs_module_context *ntvfs,
 	return NT_STATUS_INVALID_LEVEL;
 }
 
+/*
+   NTVFS fsinfo any to generic mapper
+*/
+NTSTATUS ntvfs_map_fsinfo(struct ntvfs_module_context *ntvfs,
+			  struct ntvfs_request *req,
+			  union smb_fsinfo *fs)
+{
+	NTSTATUS status;
+	union smb_fsinfo *fs2;
+
+	fs2 = talloc(req, union smb_fsinfo);
+	if (fs2 == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	if (fs->generic.level == RAW_QFS_GENERIC) {
+		return NT_STATUS_INVALID_LEVEL;
+	}
+
+	status = ntvfs_map_async_setup(ntvfs, req, fs, fs2,
+				       (second_stage_t)ntvfs_map_fsinfo_finish);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
+
+	/* ask the backend for the generic info */
+	fs2->generic.level = RAW_QFS_GENERIC;
+
+	status = ntvfs->ops->fsinfo(ntvfs, req, fs2);
+	return ntvfs_map_async_finish(req, status);
+}
+
 
 /* 
    NTVFS fileinfo generic to any mapper
@@ -920,6 +937,22 @@ NTSTATUS ntvfs_map_fileinfo(TALLOC_CTX *mem_ctx,
 }
 
 /* 
+   NTVFS any to fileinfo mapper
+*/
+static NTSTATUS ntvfs_map_qfileinfo_finish(struct ntvfs_module_context *ntvfs,
+				      struct ntvfs_request *req,
+				      union smb_fileinfo *info,
+				      union smb_fileinfo *info2,
+				      NTSTATUS status)
+{
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
+
+	return ntvfs_map_fileinfo(req, info, info2);
+}
+
+/*
    NTVFS fileinfo generic to any mapper
 */
 NTSTATUS ntvfs_map_qfileinfo(struct ntvfs_module_context *ntvfs,
@@ -938,17 +971,33 @@ NTSTATUS ntvfs_map_qfileinfo(struct ntvfs_module_context *ntvfs,
 		return NT_STATUS_INVALID_LEVEL;
 	}
 
+	status = ntvfs_map_async_setup(ntvfs, req, info, info2,
+				       (second_stage_t)ntvfs_map_qfileinfo_finish);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
+
 	/* ask the backend for the generic info */
 	info2->generic.level = RAW_FILEINFO_GENERIC;
 	info2->generic.in.file.ntvfs= info->generic.in.file.ntvfs;
 
-	/* only used by the simple backend, which doesn't do async */
-	req->async_states->state &= ~NTVFS_ASYNC_STATE_MAY_ASYNC;
-
 	status = ntvfs->ops->qfileinfo(ntvfs, req, info2);
+	return ntvfs_map_async_finish(req, status);
+}
+
+/*
+   NTVFS any to fileinfo mapper
+*/
+static NTSTATUS ntvfs_map_qpathinfo_finish(struct ntvfs_module_context *ntvfs,
+				      struct ntvfs_request *req,
+				      union smb_fileinfo *info,
+				      union smb_fileinfo *info2,
+				      NTSTATUS status)
+{
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
+
 	return ntvfs_map_fileinfo(req, info, info2);
 }
 
@@ -971,18 +1020,18 @@ NTSTATUS ntvfs_map_qpathinfo(struct ntvfs_module_context *ntvfs,
 		return NT_STATUS_INVALID_LEVEL;
 	}
 
+	status = ntvfs_map_async_setup(ntvfs, req, info, info2,
+				       (second_stage_t)ntvfs_map_qpathinfo_finish);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
+
 	/* ask the backend for the generic info */
 	info2->generic.level		= RAW_FILEINFO_GENERIC;
 	info2->generic.in.file.path	= info->generic.in.file.path;
 
-	/* only used by the simple backend, which doesn't do async */
-	req->async_states->state &= ~NTVFS_ASYNC_STATE_MAY_ASYNC;
-
 	status = ntvfs->ops->qpathinfo(ntvfs, req, info2);
-	if (!NT_STATUS_IS_OK(status)) {
-		return status;
-	}
-	return ntvfs_map_fileinfo(req, info, info2);
+	return ntvfs_map_async_finish(req, status);
 }
 
 
@@ -1059,6 +1108,9 @@ NTSTATUS ntvfs_map_lock(struct ntvfs_module_context *ntvfs,
 		/* only the first lock gives the UNLOCK bit - see
 		   MS-SMB2 3.3.5.14 */
 		if (lck->smb2.in.locks[0].flags & SMB2_LOCK_FLAG_UNLOCK) {
+			if (lck->smb2.in.locks[0].flags & SMB2_LOCK_FLAG_FAIL_IMMEDIATELY) {
+				return NT_STATUS_INVALID_PARAMETER;
+			}
 			lck2->generic.in.ulock_cnt = lck->smb2.in.lock_count;
 			isunlock = true;
 		} else {
@@ -1066,6 +1118,15 @@ NTSTATUS ntvfs_map_lock(struct ntvfs_module_context *ntvfs,
 			isunlock = false;
 		}
 		for (i=0;i<lck->smb2.in.lock_count;i++) {
+			if (!isunlock &&
+			    lck->smb2.in.locks[i].flags == SMB2_LOCK_FLAG_NONE) {
+				return NT_STATUS_INVALID_PARAMETER;
+			}
+
+			if (lck->smb2.in.locks[i].flags & ~SMB2_LOCK_FLAG_ALL_MASK) {
+				return NT_STATUS_INVALID_PARAMETER;
+			}
+
 			if (isunlock && 
 			    (lck->smb2.in.locks[i].flags & 
 			     (SMB2_LOCK_FLAG_SHARED|SMB2_LOCK_FLAG_EXCLUSIVE))) {
@@ -1128,7 +1189,7 @@ static NTSTATUS ntvfs_map_write_finish(struct ntvfs_module_context *ntvfs,
 {
 	union smb_lock *lck;
 	union smb_close *cl;
-	uint_t state;
+	unsigned int state;
 
 	if (NT_STATUS_IS_ERR(status)) {
 		return status;
@@ -1322,7 +1383,7 @@ NTSTATUS ntvfs_map_read(struct ntvfs_module_context *ntvfs,
 	union smb_read *rd2;
 	union smb_lock *lck;
 	NTSTATUS status;
-	uint_t state;
+	unsigned int state;
 
 	rd2 = talloc(req, union smb_read);
 	if (rd2 == NULL) {
