@@ -1,18 +1,16 @@
-#!/usr/bin/python
-
 # Unix SMB/CIFS implementation.
 # Copyright (C) Jelmer Vernooij <jelmer@samba.org> 2007
-#   
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
-#   
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-#   
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
@@ -67,7 +65,7 @@ class TdbDatabase(object):
 
 class Registry(TdbDatabase):
     """Simple read-only support for reading the Samba3 registry.
-    
+
     :note: This object uses the same syntax for registry key paths as 
         Samba 3. This particular format uses forward slashes for key path 
         separators and abbreviations for the predefined key names. 
@@ -683,7 +681,7 @@ class ParamFile(object):
         section = None
         for i, l in enumerate(open(filename, 'r').xreadlines()):
             l = l.strip()
-            if not l:
+            if not l or l[0] == '#' or l[0] == ';':
                 continue
             if l[0] == "[" and l[-1] == "]":
                 section = self._sanitize_name(l[1:-1])
@@ -769,7 +767,7 @@ class Samba3(object):
 
     def get_policy_db(self):
         return PolicyDatabase(self.libdir_path("account_policy.tdb"))
-    
+
     def get_registry(self):
         return Registry(self.libdir_path("registry.tdb"))
 
